import 'dart:convert';
import 'dart:io';

import 'package:example/mvu/model/model.dart';
import 'package:meta/meta.dart';
import 'package:path/path.dart' as path;
import 'package:path_provider/path_provider.dart';

class TodoStorage {
  final String fileName;

  TodoStorage({
    @required this.fileName,
  });

  File storageFile;

  Future<File> _initializeStorageFilePath() async {
    final appDocDir = await getApplicationDocumentsDirectory();
    return File(path.join(appDocDir.path, 'todo.json'));
  }

  Future<List<TodoItem>> load() async {
    if (storageFile == null) {
      storageFile = await _initializeStorageFilePath();
    }
    final storageContent = await storageFile.readAsString();

    return jsonDecode(storageContent)
        .map<TodoItem>((x) => TodoItem.fromJson(x))
        .toList();
  }

  Future<void> save(List<TodoItem> items) async {
    if (storageFile == null) {
      storageFile = await _initializeStorageFilePath();
    }
    final storageContent = jsonEncode(items.map((x) => x.toJson()).toList());
    await storageFile.writeAsString(storageContent);
  }
}
