import 'package:example/mvu/model/model.dart';
import 'package:flutter/foundation.dart';
import 'package:freezed_annotation/freezed_annotation.dart';
import 'package:mvu/mvu.dart';

import 'reducer.dart';
import 'state.dart';

part 'actions.freezed.dart';

@freezed
abstract class TodoListAction with _$TodoListAction {
  const factory TodoListAction.loading() = LoadingAction;
  const factory TodoListAction.loadingSucceeded({
    @required List<TodoItem> items,
  }) = LoadingSucceededAction;
  const factory TodoListAction.loadingFailed({
    @required dynamic error,
  }) = LoadingFailedAction;
  const factory TodoListAction.saving() = SavingAction;
  const factory TodoListAction.savingFailed({
    @required dynamic error,
  }) = SavingFailedAction;
  const factory TodoListAction.savingSucceeded() = SavingSucceededAction;
  const factory TodoListAction.updateItem({
    @required String itemId,
    @required bool isChecked,
  }) = UpdateItemAction;
  const factory TodoListAction.deleteItem({
    @required String itemId,
  }) = DeleteItemAction;
  const factory TodoListAction.createItem({
    @required String title,
  }) = CreateItemAction;

  static AsyncAction<TodoState> load() => TodoReducer.load();

  static AsyncAction<TodoState> createItemAndSave({
    @required String title,
  }) =>
      TodoReducer.createItemAndSave(title: title);

  static AsyncAction<TodoState> updateItemAndSave({
    @required String itemId,
    @required bool isChecked,
  }) =>
      TodoReducer.updateItemAndSave(
        itemId: itemId,
        isChecked: isChecked,
      );
}
