import 'package:example/mvu/model/model.dart';
import 'package:example/mvu/services/storage.dart';
import 'package:flutter/foundation.dart';
import 'package:freezed_annotation/freezed_annotation.dart';

part 'state.freezed.dart';

@freezed
abstract class TodoState with _$TodoState {
  const factory TodoState({
    @required TodoConfiguration configuration,
    @required TodoListState list,
  }) = _TodoState;
}

@freezed
abstract class TodoConfiguration with _$TodoConfiguration {
  const factory TodoConfiguration({
    @required TodoStorage storage,
  }) = _TodoConfiguration;
}

@freezed
abstract class TodoListState with _$TodoListState {
  const factory TodoListState.notLoaded() = NotLoadedState;
  const factory TodoListState.loading() = LoadingState;
  const factory TodoListState.loaded({
    @required List<TodoItem> items,
  }) = LoadedState;
  const factory TodoListState.loadingFailed({
    @required dynamic error,
  }) = LoadingFailedState;
  const factory TodoListState.saving({
    @required List<TodoItem> items,
  }) = SavingState;
  const factory TodoListState.saved({
    @required DateTime date,
    @required List<TodoItem> items,
  }) = SavedState;
  const factory TodoListState.savingFailed({
    @required dynamic error,
  }) = SavingFailed;
}
