import 'package:flutter/widgets.dart';
import 'package:mvu/src/store.dart';
import 'package:provider/provider.dart';

import 'proxy.dart';

typedef State MvuInitialStateBuilder<State>(BuildContext context);

class Mvu<State> extends StatelessWidget {
  final MvuInitialStateBuilder<State> initialStateBuilder;
  final Reducer<State> reducer;
  final Widget child;
  const Mvu({
    Key key,
    @required this.initialStateBuilder,
    @required this.reducer,
    @required this.child,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return ChangeNotifierProvider<MvuStore<State>>(
      create: (context) => MvuStore<State>(
        initialState: initialStateBuilder(context),
        reducer: reducer,
      ),
      child: child,
    );
  }
}

typedef Value MvuValueSelector<State, Value>(State state);

typedef Widget MvuValueWidgetBuilder<Value>(BuildContext context, Value value);

class MvuSelector<State, Value> extends StatelessWidget {
  final MvuValueSelector<State, Value> selector;
  final MvuValueWidgetBuilder<Value> builder;
  const MvuSelector({
    Key key,
    @required this.selector,
    @required this.builder,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return Selector<MvuStore<State>, Value>(
      selector: (context, manager) => selector(manager.state),
      builder: (context, value, _) => builder(context, value),
    );
  }
}

extension MvuExtensions on BuildContext {
  Value mvuSelect<State, Value>(MvuValueSelector<State, Value> selector) {
    return this.select<MvuStore<State>, Value>(
      (store) => selector(store.state),
    );
  }

  void dispatch<State>(dynamic action) {
    final store = Provider.of<MvuStore<State>>(this, listen: false);
    store.dispatch(action);
  }
}

class MvuProxy<State, ChildState> extends StatelessWidget {
  final MvuProxyStateSelector<State, ChildState> selector;
  final Widget child;
  const MvuProxy({
    Key key,
    @required this.selector,
    @required this.child,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    final parent = Provider.of<MvuStore<State>>(context, listen: false);
    return Provider<MvuStore<ChildState>>.value(
      value: MvuProxyStore<State, ChildState>(
        parent: parent,
        selector: selector,
      ),
      child: child,
    );
  }
}
